@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from django.db.models.fields import AutoField, CharField, TextField
+from django.db.models.fields import AutoField, CharField, TextField, SlugField
 from django.db.models.fields import DateField, DateTimeField, EmailField
 from django.db.models.fields import IntegerField, SmallIntegerField
 from django.db.models.fields import PositiveSmallIntegerField, PositiveIntegerField
@@ -7,6 +7,11 @@ from django.db.models.fields import FloatField, DecimalField
 from django.db.models.fields import BooleanField
 from django.db.models.fields import URLField
 
+from django.contrib.contenttypes import generic
+from django.contrib.contenttypes.models import ContentType
+
+from django.contrib.contenttypes import generic
+
 try:
     from django.db.models.fields import BigIntegerField
 except ImportError:
@@ -25,6 +30,8 @@ import sys
 #TODO: improve related models handling
 foreign_key_required = [lambda field: ('model', field.related.parent_model)]
 
+MAX_SELF_REFERENCE_LOOPS = 2
+
 def make_one(model, **attrs):
     """
     Creates a persisted instance from a given model its associated models.
@@ -65,6 +72,7 @@ default_mapping = {
 
     CharField:generators.gen_string,
     TextField:generators.gen_text,
+    SlugField:generators.gen_slug,
 
     ForeignKey:make_one,
     #OneToOneField:make_one,
@@ -75,6 +83,8 @@ default_mapping = {
 
     URLField:generators.gen_url,
     EmailField:generators.gen_email,
+
+    ContentType:generators.gen_content_type,
 }
 
 class Mommy(object):
@@ -109,9 +119,15 @@ class Mommy(object):
             ignore_field = lambda field: field.null and not self.fill_nullables
             field_value_not_defined = field.name not in attrs
 
-            if isinstance(field, AutoField):
+            if isinstance(field, (AutoField, generic.GenericRelation)):
                 continue
 
+            # If not specified, django automatically sets blank=True and default
+            # on BooleanFields so we don't need to check these
+            if not isinstance(field, BooleanField):
+                if field.has_default() or field.blank:
+                    continue
+
             if isinstance(field, ManyToManyField):
                 if field_value_not_defined:
                     if ignore_field(field):
@@ -125,8 +141,17 @@ class Mommy(object):
                 if ignore_field(field):
                     continue
                 else:
+                    #XXX: hack to avoid infinite recursion
+                    if isinstance(field, ForeignKey) and (field.related.parent_model == self.model):
+                        self.model.self_reference_loop_count = getattr(self.model, 'self_reference_loop_count', 1)
+                        if self.model.self_reference_loop_count <= MAX_SELF_REFERENCE_LOOPS:
+                            self.model.self_reference_loop_count += 1
+                        else:
+                            del self.model.self_reference_loop_count
+                            return
                     attrs[field.name] = self.generate_value(field)
 
+
         instance = self.model(**attrs)
 
         # m2m only works for persisted instances
@@ -152,17 +177,21 @@ class Mommy(object):
 
         `attr_mapping` and `type_mapping` can be defined easely overwriting the model.
         '''
+
         if field.name in self.attr_mapping:
             generator = self.attr_mapping[field.name]
         elif getattr(field, 'choices'):
             generator = generators.gen_from_choices(field.choices)
+        elif isinstance(field, ForeignKey) and field.rel.to is ContentType:
+            generator = self.type_mapping[ContentType]
         elif field.__class__ in self.type_mapping:
             generator = self.type_mapping[field.__class__]
         else:
             raise TypeError('%s is not supported by mommy.' % field.__class__)
 
-        required_fields = get_required_values(generator, field)
-        return generator(**required_fields)
+        # attributes like max_length, decimal_places are take in account when generating the value.
+        required_field_attrs = get_required_values(generator, field)
+        return generator(**required_field_attrs)
 
 def get_required_values(generator, field):
     '''
@@ -187,4 +216,3 @@ def get_required_values(generator, field):
                 raise ValueError("Required value '%s' is of wrong type. Don't make mommy sad." % str(item))
 
     return rt
-
